
import unittest

from netlookup.network import Network
from netaddr.core import AddrFormatError

from ..constants import VALID_SUBNET_VALUES, INVALID_SUBNET_VALUES, MAX_SPLITS


class TestSubnetParsing(unittest.TestCase):
    """
    Test subnet value parsing and operations
    """

    def validate_network(self, value):
        """
        Validate a network value
        """
        network = Network(value)
        return network

    def test_valid_subnet_values(self):
        """
        Test parsing of valid values for subnet object
        """
        for value in VALID_SUBNET_VALUES:
            self.validate_network(value)

    def test_invalid_subnet_values(self):
        """
        Test parsing of invalid values for subnet object
        """
        for value in INVALID_SUBNET_VALUES:
            with self.assertRaises(AddrFormatError):
                self.validate_network(value)

    def test_integer_compare_functions(self):
        """
        Test integer compare functions for extended subnet objects
        """
        for value in VALID_SUBNET_VALUES:
            network = self.validate_network(value)
            self.assertTrue(network == network.value)
            self.assertTrue(network != network.value - 1)
            self.assertTrue(network < network.last + 1)
            self.assertTrue(network <= network.last)
            self.assertTrue(network > network.value - 1)
            self.assertTrue(network >= network.value)

    def test_last_host_property(self):
        """
        Test custom last_host property
        """
        for value in VALID_SUBNET_VALUES:
            network = self.validate_network(value)
            if network.version == 4 and network.prefixlen == 31:
                self.assertEqual(network.last_host.value, network.last)
            elif network.version == 4 and network.prefixlen == 32:
                self.assertIsNone(network.last_host)
            elif network.version == 6 and network.prefixlen == 127:
                self.assertEqual(network.last_host.value, network.last)
            elif network.version == 6 and network.prefixlen == 128:
                self.assertIsNone(network.last_host)
            else:
                self.assertEqual(network.last_host.value, network.last - 1)

    def test_first_host_property(self):
        """
        Test custom last_host property
        """
        for value in VALID_SUBNET_VALUES:
            network = self.validate_network(value)
            if network.version == 4 and network.prefixlen == 32:
                self.assertIsNone(network.first_host)
            elif network.version == 4 and network.prefixlen == 31:
                self.assertEqual(network.first_host.value, network.first)
            elif network.version == 6 and network.prefixlen == 127:
                self.assertEqual(network.first_host.value, network.first)
            elif network.version == 6 and network.prefixlen == 128:
                self.assertIsNone(network.first_host)
            else:
                self.assertEqual(network.first_host.value, network.first + 1)

    def test_splitting_v4_subnets(self):
        """
        Test splitting subnet
        """
        for value in VALID_SUBNET_VALUES:
            network = self.validate_network(value)
            if network.version == 4:
                prefixlen = network.prefixlen + 1
                if prefixlen == 0:
                    prefixlen += 1
                while prefixlen < 32 and prefixlen < network.prefixlen + MAX_SPLITS:
                    testdata = list(network.subnet(prefixlen))
                    self.assertTrue(len(testdata) > 0)
                    prefixlen += 1

    def test_splitting_v6_subnets(self):
        """
        Test splitting IPv4 subnets
        """
        for value in VALID_SUBNET_VALUES:
            network = self.validate_network(value)
            if network.version == 6:
                prefixlen = network.prefixlen + 1
                if prefixlen == 0:
                    prefixlen += 1
                while prefixlen < 128 and prefixlen < network.prefixlen + MAX_SPLITS:
                    testdata = list(network.subnet(prefixlen))
                    self.assertTrue(len(testdata) > 0)
                    prefixlen += 1




import unittest

from netlookup.network import Network
from netlookup.prefixes import NetworkSet, NetworkSetError

from ..constants import VALID_SUBNET_VALUES, INVALID_SUBNET_VALUES, MAX_SPLITS


class TestNetworkSets(unittest.TestCase):
    """
    Test cases for network sets
    """

    def test_network_set_create_empty(self):
        """
        Create empty network set with various arguments
        """
        network_set = NetworkSet()
        self.assertIsInstance(network_set.networks, list)

    def test_network_set_add_valid_networks(self):
        """
        Test loading valid network prefixes to empty network set
        """
        network_set = NetworkSet()
        for network in VALID_SUBNET_VALUES:
            network_set.add_network(network)
        self.assertEqual(len(network_set.networks), len(VALID_SUBNET_VALUES))

    def test_network_set_add_invalid_networks(self):
        """
        Test loading invalid network prefixes to empty network set
        """
        network_set = NetworkSet()
        for network in INVALID_SUBNET_VALUES:
            with self.assertRaises(NetworkSetError):
                network_set.add_network(network)
        network_count = len(network_set.networks)
        self.assertEqual(network_count, 0, 'Expected empty set, has {} networks'.format(network_count))

    def test_network_set_merged_empty(self):
        """
        Teste 'merged' property for empty network set
        """
        network_set = NetworkSet()
        self.assertEqual(network_set.merged, [])

    def test_network_set_merge_valid_networks(self):
        """
        Test merging valid network ranges
        """
        network_set = NetworkSet()
        expected = 0
        for network in VALID_SUBNET_VALUES:
            # Ignore global masks
            if Network(network).prefixlen == 0:
                continue
            network_set.add_network(network)
            expected += 1
        merged = network_set.merged
        self.assertEqual(
            len(merged),
            expected,
            'Expected {} merged networks, got {}'.format(expected, merged)
        )

    def test_network_set_merge_split_ranges(self):
        """
        Test merging split IP ranges

        Splits the range to smaller units and ensures the network set still only contains
        the original split network
        """
        network_set = NetworkSet()
        network = Network('172.31.4.0/22')
        network_set.add_network(network)

        network_count = 1
        for i in range(1, MAX_SPLITS + 1):
            prefixlen = network.prefixlen + i
            for subnet in network.subnet(prefixlen):
                network_set.add_network(subnet)
                network_count += 1
                self.assertEqual(len(network_set.networks), network_count)
                self.assertTrue(
                    len(network_set.merged) == 1,
                    'Expected 1 merged network after splitting {} by prefixlen {}'.format(
                        network,
                        prefixlen
                    )
                )




from systematic.shell import Script

from .commands.info import Info
from .commands.split import Split


def main():
    script = Script()

    script.add_subcommand(Info())
    script.add_subcommand(Split())

    script.run()




import re

from datetime import datetime
from dns import resolver
from operator import attrgetter

from ..prefixes import NetworkSet, NetworkSetError, NetworkSetItem

ADDRESS_LIST_RECORD = '_cloud-netblocks.googleusercontent.com'
RE_INCLUDE = re.compile(r'^include:(?P<rr>.*)$')
RE_IPV4 = re.compile(r'^ip4:(?P<prefix>.*)$')
RE_IPV6 = re.compile(r'^ip6:(?P<prefix>.*)$')


class GCPPrefix(NetworkSetItem):
    """
    GCP cloud network prefix
    """
    type = 'gcp'


class GCP(NetworkSet):
    """
    Google computing platform address ranges

    Note: this data currently does not include information about regions
    """
    type = 'gcp'
    cache_filename = 'gcp-networks.json'
    loader_class = GCPPrefix

    def fetch(self):
        def process_rr(rr, includes):
            res = resolver.query(rr, 'TXT')
            try:
                data = str(res.rrset[0].strings[0], 'utf-8')
            except Exception as e:
                raise NetworkSetError('Error parsing data for {}: {}'.format(rr, e))

            for field in data.split(' '):
                m = RE_IPV4.match(field)
                if m:
                    self.networks.append(self.loader_class(m.groupdict()['prefix']))
                    continue

                m = RE_IPV6.match(field)
                if m:
                    self.networks.append(self.loader_class(m.groupdict()['prefix']))
                    continue

                m = RE_INCLUDE.match(field)
                if m:
                    include = m.groupdict()['rr']
                    if include not in includes:
                        process_rr(include, includes)
                    includes.append(include)
                    continue

        self.networks = []
        includes = []
        process_rr(ADDRESS_LIST_RECORD, includes)

        self.updated = datetime.now()
        self.networks.sort(key=attrgetter('version', 'cidr'))




from netaddr.ip import IPNetwork, IPAddress
from netaddr.core import AddrFormatError


class Network(IPNetwork):
    """
    Extend IPSubnet with some custom attributes
    """

    def __eq__(self, other):
        if isinstance(other, Network):
            return self.value == other.value and self.prefixlen == other.prefixlen
        if isinstance(other, IPAddress):
            return self.value == other.value
        elif isinstance(other, int):
            return self.value == other

    def __ne__(self, other):
        return not self.__eq__(other)

    def __lt__(self, other):
        if isinstance(other, Network) or isinstance(other, IPAddress):
            return self.value < other.value
        elif isinstance(other, int):
            return self.value < other

    def __le__(self, other):
        if isinstance(other, Network) or isinstance(other, IPAddress):
            return self.value <= other.value
        elif isinstance(other, int):
            return self.value <= other

    def __gt__(self, other):
        if isinstance(other, Network) or isinstance(other, IPAddress):
            return self.last > other.value
        elif isinstance(other, int):
            return self.value > other

    def __ge__(self, other):
        if isinstance(other, Network) or isinstance(other, IPAddress):
            return self.value >= other.value
        elif isinstance(other, int):
            return self.value >= other

    @property
    def total_hosts(self):
        """
        Return total number of available hosts in subnet, excluding network and broadcast
        addresses
        """
        if self.version == 4:
            if self.prefixlen == 31:
                return 2
            if self.prefixlen == 32:
                return 1
        if self.version == 6:
            if self.prefixlen == 127:
                return 2
            if self.prefixlen == 128:
                return 1
        return self.size - 2

    @property
    def first_host(self):
        """
        Return first available host in network, excluding network address
        """
        if self.version == 4:
            if self.prefixlen == 31:
                return IPAddress(self.first)
            if self.prefixlen == 32:
                return None
        if self.version == 6:
            if self.prefixlen == 127:
                return IPAddress(self.first)
            if self.prefixlen == 128:
                return None
        return IPAddress(self.first + 1)

    @property
    def last_host(self):
        """
        Return last available host in network, excluding broadcast address
        """
        if self.version == 4:
            if self.prefixlen == 31:
                return IPAddress(self.last)
            if self.prefixlen == 32:
                return None
        if self.version == 6:
            if self.prefixlen == 127:
                return IPAddress(self.last)
            if self.prefixlen == 128:
                return None
        return IPAddress(self.last - 1)

    @property
    def next_subnet_prefix(self):
        """
        Return next subnet prefix size, i.e. smaller prefix than this

        May return None when item is already a host only network
        """
        if self.version == 4:
            if self.prefixlen == 32:
                return None
        if self.version == 6:
            if self.prefixlen == 128:
                return None
        return self.prefixlen + 1

    @property
    def parent_subnet_prefix(self):
        """
        Return parent subnet prefix size, i.e. one larger prefix than this

        May return None when item is already 0
        """
        if self.prefixlen == 0:
            return None
        return self.prefixlen - 1

    def subnet(self, prefixlen):
        """
        Return subnets split by specified prefix

        Adds extra validation for prefixlen, checking the value is not out off scope
        based on address type
        """
        prefixlen = int(prefixlen)
        if self.version == 4:
            if prefixlen < 0 or prefixlen > 32:
                raise AddrFormatError('Invalid IPv4 prefix len')

        if self.version == 6:
            if prefixlen < 0 or prefixlen > 128:
                raise AddrFormatError('Invalid IPv4 prefix len')

        if prefixlen <= self.prefixlen:
            raise AddrFormatError('Split mask {} is not valid for prefixlen {}'.format(
                prefixlen,
                self.prefixlen
            ))

        return super().subnet(prefixlen)




import json
import os
import tempfile
import unittest

from netlookup.prefixes import CloudNetworks

DATA_PATH = os.path.join(os.path.dirname(__file__), 'data')
TEST_DATA_PREFIX_LEN = 25712
TEST_NETWORK_ADDRESSES = (
    {
        'address': '8.34.208.0',
        'network': '8.34.208.0/20',
    },
    {
        'address': '8.34.223.10',
        'network': '8.34.208.0/20',
    },
    {
        'address': '8.34.223.254',
        'network': '8.34.208.0/20',
    },
    {
        'address': '8.34.223.255',
        'network': '8.34.208.0/20',
    },
    {
        'address': '8.34.224.0',
    },
    {
        'address': '2c0f:fb50:4000::0',
        'network': '2c0f:fb50:4000::/36'
    },
    {
        'address': '2c0f:fb50:4000::10',
        'network': '2c0f:fb50:4000::/36'
    },
    {
        'address': '2c0f:fb50:4fff:ffff:ffff:ffff:ffff:fffe',
        'network': '2c0f:fb50:4000::/36'
    },
    {
        'address': '2c0f:fb50:4fff:ffff:ffff:ffff:ffff:ffff',
        'network': '2c0f:fb50:4000::/36'
    },
    {
        'address': '2c0f:fb50:5000::0'
    }
)
TEST_JSON_REQUIRED_FIELDS = ('type', 'cidr')
TEST_FILTER_COUNTS = {
    'invalidvalue': 0,
    'aws': 1360,
    'azure': 14621,
    'gcp': 9556,
    'google': 175,
}


class TestCloudVendorPrefixes(unittest.TestCase):
    """
    Test loading prefix data from a snapshot of actual data
    """

    def setUp(self) -> None:
        self.tempdir = None

    def tearDown(self) -> None:
        """
        Cleanup temporary test directory after test
        """
        if self.tempdir is not None and os.path.isdir(self.tempdir):
            try:
                for filename in os.listdir(self.tempdir):
                    os.unlink(os.path.join(self.tempdir, filename))
            except Exception as e:
                print('Error removing files in temporary directory {}: {}'.format(self.tempdir, e))

            try:
                os.rmdir(self.tempdir)
            except Exception as e:
                print('Error removing temporary directory {}: {}'.format(self.tempdir, e))

    def test_cloud_vendor_networks_no_data(self) -> None:
        """
        Test creating networks with no data using new temporary directory
        """
        self.tempdir = tempfile.mkdtemp()
        data = CloudNetworks(cache_directory=self.tempdir)
        self.assertEqual(len(data.networks), 0)

    def disabled_test_cloud_vendor_networks_updating(self) -> None:
        """
        Test updating test data from network. This test case requires internet connection
        """
        self.tempdir = tempfile.mkdtemp()
        data = CloudNetworks(cache_directory=self.tempdir)
        data.update()
        self.assertFalse(len(data.networks) == 0)

    def test_cloud_vendor_networks_loading(self) -> None:
        """
        Test creating networks with test data, ensuring correct number of networks is loaded
        """
        data = CloudNetworks(cache_directory=DATA_PATH)
        self.assertEqual(len(data.networks), TEST_DATA_PREFIX_LEN)

    def test_cloud_vendor_prefix_filtering(self):
        """
        Test filtering networks
        """
        data = CloudNetworks(cache_directory=DATA_PATH)
        for name, count in TEST_FILTER_COUNTS.items():
            matches = data.filter_type(name)
            self.assertEqual(len(matches), count, 'Mismatch in {} count {} != {}'.format(
                name,
                len(matches),
                count
            ))

    def test_cloud_vendor_prefix_lookup(self):
        """
        Test looking up networks from test data
        """
        data = CloudNetworks(cache_directory=DATA_PATH)
        for testcase in TEST_NETWORK_ADDRESSES:
            prefix = data.find(testcase['address'])
            network = testcase.get('network', None)
            if network is not None:
                self.assertIsNotNone(prefix, 'Error looking up address {}'.format(testcase['address']))
                self.assertEqual(str(prefix.cidr), network)
            else:
                self.assertIsNone(prefix)

    def test_cloud_vendor_json_formatting(self):
        """
        Test all loaded networks can be formatted as JSON
        """
        data = CloudNetworks(cache_directory=DATA_PATH)
        for prefix in data.networks:
            testdata = json.loads(json.dumps(prefix.as_dict()))
            for field in TEST_JSON_REQUIRED_FIELDS:
                self.assertTrue(field in testdata)
                self.assertIsNotNone(testdata[field])




from .base import BaseCommand


class Info(BaseCommand):

    name = 'info'
    short_description = 'Show subnet info'

    def print_network_details(self, network, args):
        """
        Show details for specified network
        """
        first = network.first_host
        last = network.last_host
        self.message('         CIDR {}'.format(network.cidr))
        self.message('      Netmask {}'.format(network.netmask))
        self.message('      Network {}'.format(network.network))
        if network.broadcast is not None:
            self.message('    Broadcast {}'.format(network.broadcast))
        if first is not None:
            self.message('   First host {}'.format(network.first_host))
        if last is not None:
            self.message('    Last host {}'.format(network.last_host))
        self.message('  Total hosts {}'.format(network.total_hosts))
        self.message('         Next {}'.format(network.next())) # noqa B305
        self.message('     Previous {}'.format(network.previous()))
        self.message('         Bits {}'.format(network.network.bits()))
        self.message('  Reverse DNS {}'.format(network.network.reverse_dns))

    def __register_arguments__(self, parser):
        parser.add_argument('subnets', nargs='*', help='Subnets to process')

    def run(self, args):
        if not args.subnets:
            self.exit(1, 'No subnets specified')

        for network in self.networks:
            self.print_network_details(network, args)




import json
import requests

from datetime import datetime
from operator import attrgetter

from ..prefixes import NetworkSet, NetworkSetError, NetworkSetItem

AZURE_SERVICES_URL = 'https://download.microsoft.com/download/7/1/D/71D86715-5596-4529-9B13-DA13A5DE5B63/ServiceTags_Public_20190603.json' # noqa E501


class AzurePrefix(NetworkSetItem):
    """
    Azure network prefix with region and service details
    """
    type = 'azure'
    extra_attributes = ('id', 'name', 'service', 'region')

    def __init__(self, network, data=None):
        self.id = None
        self.name = None
        self.region = None
        self.service = None
        super().__init__(network, data)

    def __repr__(self):
        return '{} {} {}'.format(self.type, self.name, self.cidr)

    def as_dict(self):
        """
        Return azure prefix with extra details
        """
        data = super().as_dict()
        data.update({
            'id': self.id,
            'name': self.name,
            'service': self.service,
            'region': self.region,
        })
        return data


class Azure(NetworkSet):
    """
    Azure address networks
    """
    type = 'azure'
    cache_filename = 'azure-networks.json'
    loader_class = AzurePrefix

    def fetch(self):
        """
        Fetch Azure IP range data
        """
        try:
            res = requests.get(AZURE_SERVICES_URL)
            if res.status_code != 200:
                raise ValueError('HTTP status code {}'.format(res.status_code))
        except Exception as e:
            raise NetworkSetError('Error fetching AWS IP ranges: {}'.format(e))

        try:
            data = json.loads(res.content)
        except Exception as e:
            raise NetworkSetError('Error loading AWS IP range data: {}'.format(e))

        self.networks = []
        for section in data['values']:
            kwargs = {
                'id': section['id'],
                'name': section['name'],
                'service': section['properties']['systemService'],
                'region': section['properties']['region'],
            }
            for prefix in section['properties']['addressPrefixes']:
                self.networks.append(self.loader_class(prefix, kwargs))

        self.updated = datetime.now()
        self.networks.sort(key=attrgetter('version', 'region', 'service'))




import json
import os
import sys

from bisect import bisect_left
from datetime import datetime
from netaddr.ip import IPAddress
from netaddr.ip.sets import IPSet
from operator import attrgetter

from .network import Network

if sys.platform == 'darwin':
    CACHE_DIR = os.path.expanduser('~/Library/Caches/netlookup')
else:
    CACHE_DIR = os.path.expanduser('~/.cache/netlookup')


class NetworkSetError(Exception):
    pass


class NetworkSetItem(Network):
    """
    Named network prefix for specific vendor and service
    """
    type = 'generic'
    extra_attributes = []

    def __init__(self, network, data=None):
        super().__init__(network)

        if data is not None:
            for attr in self.extra_attributes:
                if attr in data:
                    setattr(self, attr, data[attr])

    def __repr__(self):
        return '{} {}'.format(self.type, self.cidr)

    def __str__(self):
        return self.__repr__()

    def as_dict(self):
        """
        Format prefix object as dictionary

        Extend in child class to get prefix type specific data
        """
        data = {
            'type': self.type,
            'cidr': str(self.cidr)
        }
        for attr in self.extra_attributes:
            data[attr] = getattr(self, attr)
        return data


class NetworkSet:
    """
    Common base class for network address prefix sets with caching
    """
    type = 'generic'
    cache_filename = None
    loader_class = NetworkSetItem

    def __init__(self, cache_directory=CACHE_DIR):
        self.cache_directory = cache_directory
        self.updated = None
        self.networks = []
        self.__iter_index__ = None

        if not os.path.isdir(CACHE_DIR):
            try:
                os.makedirs(CACHE_DIR)
            except Exception as e:
                raise NetworkSetError('Error creating directory {}: {}'.format(CACHE_DIR, e))

        if self.cache_file is not None and os.path.isfile(self.cache_file):
            self.load()

    def __iter__(self):
        return self

    def __next__(self):
        if not self.networks:
            self.fetch()
        if self.__iter_index__ is None:
            self.__iter_index__ = 0
        try:
            item = self.networks[self.__iter_index__]
            self.__iter_index__ += 1
            return item
        except IndexError:
            self.__iter_index__ = None
            raise StopIteration

    @property
    def cache_file(self):
        """
        Filename for prefix data cache file
        """
        if self.cache_filename is not None:
            return os.path.join(self.cache_directory, self.cache_filename)
        else:
            return None

    @property
    def ipset(self):
        """
        IPSet of networks in network set
        """
        return IPSet(self.networks)

    @property
    def merged(self):
        """
        Minimal merged set of IP range set covering network set
        """
        return [self.loader_class(network) for network in self.ipset.iter_cidrs()]

    def fetch(self):
        raise NotImplementedError('fetch() must be implemented in child class')

    def as_dict(self):
        """
        Return all networks as dictionary
        """
        return {
            'updated': self.updated.isoformat() if self.updated else None,
            'networks': [prefix.as_dict() for prefix in self.networks]
        }

    def add_network(self, value):
        """
        Add network to cache
        """
        try:
            network = self.loader_class(value)
        except Exception as e:
            raise NetworkSetError('Error parsing network {}: ()'.format(value, e))
        if network not in self.networks:
            self.networks.append(network)

    def substract(self, networks):
        """
        Return merged network set, with specified network removed
        """
        ipset = self.ipset
        if isinstance(networks, str):
            networks = [networks]
        for network in networks:
            ipset.remove(network)
        return [self.loader_class(network) for network in ipset.iter_cidrs()]

    def load(self):
        """
        Load local cache file
        """
        if self.cache_file is None or not os.path.isfile(self.cache_file):
            return

        self.networks = []
        try:
            with open(self.cache_file, 'r') as fd:
                data = json.loads(fd.read())
        except Exception as e:
            raise NetworkSetError('Error reading cache file {}: {}'.format(
                self.cache_file,
                e
            ))

        try:
            self.updated = datetime.fromisoformat(data['updated'])
            for record in data['networks']:
                prefix = self.loader_class(record['cidr'], record)
                self.networks.append(prefix)
        except Exception as e:
            raise NetworkSetError('Error loading data from cache file {}: {}'.format(
                self.cache_file,
                e
            ))

    def save(self):
        """
        Save data to cache file
        """
        if self.cache_file is None:
            return

        try:
            with open(self.cache_file, 'w') as fd:
                fd.write('{}\n'.format(json.dumps(self.as_dict(), indent=2)))
        except Exception as e:
            raise NetworkSetError('Error writing cache file {}: {}'.format(
                self.cache_file,
                e
            ))

    def find(self, value):
        """
        Find address in networks
        """
        address = IPAddress(value)
        next = bisect_left(self.networks, address)
        if next > 0:
            prefix = self.networks[next - 1]
            if address == prefix or address in prefix:
                return prefix
        return None


class CloudNetworks:
    """
    Loader and lookup for known IP address prefix caches for public clouds
    """
    def __init__(self, cache_directory=CACHE_DIR):
        self.cache_directory = cache_directory
        from .cloud.aws import AWS
        from .cloud.azure import Azure
        from .cloud.gcp import GCP
        from .cloud.google import GoogleServices

        self.vendors = []
        self.networks = []

        self.add_vendor(AWS(cache_directory=self.cache_directory))
        self.add_vendor(Azure(cache_directory=self.cache_directory))
        self.add_vendor(GCP(cache_directory=self.cache_directory))
        self.add_vendor(GoogleServices(cache_directory=self.cache_directory))

        self.load()

    def add_vendor(self, vendor):
        """
        Add vendor to setup
        """
        self.vendors.append(vendor)

    def update(self):
        """
        Fetch and update cached prefix data
        """
        for vendor in self.vendors:
            try:
                vendor.fetch()
                vendor.save()
            except Exception as e:
                raise NetworkSetError('Error updating {} data: {}'.format(vendor, e))
        self.load()

    def save(self):
        """
        Save cached data for vendors
        """
        for vendor in self.vendors:
            vendor.save()

    def load(self):
        """
        Load cached networks
        """
        self.networks = []
        for vendor in self.vendors:
            vendor.load()
            for prefix in vendor.networks:
                self.networks.append(prefix)
        self.networks.sort(key=attrgetter('value'))

    def filter_type(self, value):
        """
        Filter networks by type
        """
        return [prefix for prefix in self.networks if prefix.type == value]

    def get_vendor(self, name):
        """
        Get vendor prefix set
        """
        for vendor in self.vendors:
            if vendor.type == name:
                return vendor
        raise NetworkSetError('No such vendor: {}'.format(name))

    def find(self, value):
        """
        Find address in networks
        """
        address = IPAddress(value)
        next = bisect_left(self.networks, address)
        if next > 0:
            try:
                prefix = self.networks[next]
                if address.value == prefix.value or address in prefix:
                    return prefix
            except IndexError:
                pass
            prefix = self.networks[next - 1]
            if address.value == prefix.value or address in prefix:
                return prefix
        return None




import json
import requests

from datetime import datetime
from operator import attrgetter

from ..prefixes import NetworkSet, NetworkSetError, NetworkSetItem

AWS_IP_RANGES_URL = 'https://ip-ranges.amazonaws.com/ip-ranges.json'
SKIP_SERVICE_NAMES = (
    'AMAZON',
)


class AWSPrefix(NetworkSetItem):
    """
    AWS network prefix with region and service details
    """
    type = 'aws'
    extra_attributes = ('region', 'services')

    def __init__(self, network, data=None):
        self.region = None
        self.services = []
        super().__init__(network, data)

    def __repr__(self):
        return '{} {} {}'.format(self.type, self.region, self.cidr)


class AWS(NetworkSet):
    """
    AWS address networks
    """
    type = 'aws'
    cache_filename = 'aws-networks.json'
    loader_class = AWSPrefix

    @property
    def regions(self):
        """
        Return all detected regions
        """
        return sorted(set(prefix.region for prefix in self.networks))

    def fetch(self):
        """
        Fetch AWS IP range data
        """

        try:
            res = requests.get(AWS_IP_RANGES_URL)
            if res.status_code != 200:
                raise ValueError('HTTP status code {}'.format(res.status_code))
        except Exception as e:
            raise NetworkSetError('Error fetching AWS IP ranges: {}'.format(e))

        try:
            data = json.loads(res.content)
        except Exception as e:
            raise NetworkSetError('Error loading AWS IP range data: {}'.format(e))

        self.updated = datetime.fromtimestamp(int(data['syncToken']))

        networks = {}
        for item in data['prefixes']:
            prefix = self.loader_class(item['ip_prefix'], item)
            if prefix.cidr not in networks:
                networks[prefix.cidr] = prefix
            if item['service'] not in SKIP_SERVICE_NAMES and item['service'] not in networks[prefix.cidr].services:
                networks[prefix.cidr].services.append(item['service'])

        for item in data['ipv6_prefixes']:
            prefix = self.loader_class(item['ipv6_prefix'], item)
            if prefix.cidr not in networks:
                networks[prefix.cidr] = prefix
            if item['service'] not in SKIP_SERVICE_NAMES and item['service'] not in networks[prefix.cidr].services:
                networks[prefix.cidr].services.append(item['service'])

        self.networks = list(networks.values())
        self.networks.sort(key=attrgetter('version', 'region', 'services', 'cidr'))




from systematic.shell import ScriptCommand

from ...network import Network


class BaseCommand(ScriptCommand):

    def parse_args(self, args):
        self.networks = []
        if 'subnets' in args:
            try:
                for arg in args.subnets:
                    self.networks.append(Network(arg))
            except Exception as e:
                self.exit(1, 'Error parsing subnets: {}'.format(e))

        return args




from .base import BaseCommand


class Split(BaseCommand):

    name = 'split'
    short_description = 'Show split subnet to prefixes'

    def __register_arguments__(self, parser):
        parser.add_argument('-s', '--subnet-prefix', type=int, help='Prefix length for split networks')
        parser.add_argument('subnets', nargs='*', help='Subnets to process')

    def run(self, args):
        if not args.subnets:
            self.exit(1, 'No subnets specified')

        for network in self.networks:
            prefixlen = args.subnet_prefix if args.subnet_prefix is not None else network.next_subnet_prefix
            if prefixlen is not None:
                try:
                    for subnet in network.subnet(prefixlen):
                        self.message(subnet)
                except Exception as e:
                    self.error('Error splitting {}: {}'.format(network, e))
            else:
                self.error('Network is not splittable {}'.format(network))
